unit UpMain;

interface

uses
  Windows, Messages, SysUtils, Classes, HTTPApp, IniFiles, PostContent;

type
  TWebModul = class(TWebModule)
    Answer: TPageProducer;
    procedure WebModule1WebActionAction(Sender: TObject;
      Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
    procedure WebModuleAfterDispatch(Sender: TObject; Request: TWebRequest;
      Response: TWebResponse; var Handled: Boolean);
    procedure AnswerHTMLTag(Sender: TObject; Tag: TTag;
      const TagString: String; TagParams: TStrings; var ReplaceText: String);
  private
    sMsg, HTMLAnsw: String;
  end;

var
  WebModul: TWebModul;

implementation

{$R *.DFM}

{ ----Hilfs-Routinen -------------------------------------------------------- }

function GetIniName: String;            // Name der zugeordneten Parameterdatei
begin GetIniName:=ChangeFileExt(Paramstr(0),'.ini') end;

function DeCrypt(const sIn: String): String;                 // Entschlsselung
begin
 { hier bei Bedarf eigenen Entschlsslungscode eintragen                      }
 DeCrypt:=sIn;                                  // z.Z. unverndert zurckgeben
end;

{---- Hauptprogramm ----------------------------------------------------------}

procedure TWebModul.WebModule1WebActionAction(Sender: TObject;
           Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
var Ini: TIniFile;                          // Parameterdatei zur Konfiguration
    Liste: TStringList;                  // Ergebnisliste fr Formularfeldwerte
    UpFile: TMemoryStream;                           // Puffer fr Antwortseite
    sResult, sTempDir, sUserDir, sFile, sUser: String;
    cBuf: array[0..254] of Char;
begin
 Ini:=TIniFile.Create(GetIniName);
 sTempDir:=  Ini.ReadString('Settings','TempDir','');   // temporres Verzeichnis
 HTMLAnsw:=Ini.ReadString('Settings','Answer',''); //Schablone fr Ergeb.-seite
 Ini.Free;
 if sTempDir='' then begin                    // nichts in INI-Datei vorgegeben
  FillChar(cBuf,SizeOf(cBuf),#0);
  if GetEnvironmentVariable('TEMP',cBuf,SizeOf(cBuf))>1 then sTempDir:=cBuf
  else if GetEnvironmentVariable('TMP',cBuf,SizeOf(cBuf))>1 then sTempDir:=cBuf;
 end;      // wenn nichts angegeben, wird halt das aktuelle Verzeichnis benutzt
 if sTempDir<>'' then if sTempDir[Length(sTempDir)]<>'\' then sTempDir:=sTempDir+'\';
 sTempDir:=sTempDir+IntToStr(Abs(GetCurrentThreadID))+'\'; // wegen gleichzeitigen Usern
 try MkDir(sTempDir) except end;               // eindeutiges Pufferverzeichnis
 Liste:=TStringList.Create;                 // Pufferspeicher fr Formulardaten
 if GetPostedContent(Request,Liste,sTempDir) then begin  // gePOSTete Daten aufgenommen
  sFile:=ExtractFileName(Liste.Values['upfile']); //Dateiname aus Formulardaten
  if sFile<>'' then begin
   sUser:=Liste.Values['user'];
   if sUser<>'' then begin             // Username ist ins Formular eingetragen
    Ini:=TIniFile.Create(GetIniName);
    sResult:=DeCrypt(Ini.ReadString(sUser,'Pass','@')); // Passwort aus der Datenbank
    sUserDir:=Ini.ReadString(sUser,'UserDir','');     // privates Speicherverzeichnis
    Ini.Free;
    if sResult=Liste.Values['pass'] then begin // mit Passwort aus Formular vergleichen
     if (sUserDir<>'') then begin    // privates Speicherverzeichnis ist gltig
      if sUserDir[Length(sUserDir)]<>'\' then sUserDir:=sUserDir+'\';
      UpFile:=TMemoryStream.Create;
      try                                      // hochgeladene Datei umkopieren
       UpFile.LoadFromFile(sTempDir+sFile); UpFile.SaveToFile(sUserDir+sFile);
       sMsg:=Format('Die Datei <B>%s</B> von %s wurde im privaten Web-Verzeichnis abgelegt.',[sFile,sUser]);
      except sMsg:=Format('Die Datei <B>%s</B> von %s konnte <B>nicht</B> ordnungsgem&auml;&szlig; gespeichert werden.',[sFile,sUser]) end;
      UpFile.Free;
     end else sMsg:=Format('Ihr privates UpLoad-Verzeichnis ist <B>nicht</B> korrekt eingerichtet.',[sFile,sUser]);
    end else sMsg:='Das angegebene Benutzerpasswort ist <B>nicht</B> korrekt.';
   end else sMsg:='Es wurde <B>kein</B> Benutzername in das Formular eingetragen.';
   try DeleteFile(sTempDir+sFile) except end; // evt. hochgeladene Datei aus TEMP lschen
  end else sMsg:='Es wurde <B>keine</B> Datei zum Hochladen in das Formular eingetragen';
 end else sMsg:='Sie benutzen <B>nicht</B> das richtige Formular.';
 Liste.Free;
 try RmDir(sTempDir) except end;  // leeres Unterverzeichnis von TEMP lschen
end;

procedure TWebModul.WebModuleAfterDispatch(Sender: TObject;
  Request: TWebRequest; Response: TWebResponse; var Handled: Boolean);
begin
  if HTMLAnsw<>'' then Answer.HTMLFile:=HTMLAnsw // in Action aus INI-Datei gelesen
  else Answer.HTMLFile:=ChangeFileExt(GetIniName,'.htm'); // Defaultname versuchen
  try Response.Content:=Answer.Content        // Antwortschablone war vorhanden
  except                               // Antwortschablone war nicht zugnglich
   Answer.HTMLFile:='';
   Answer.HTMLDoc.Add('<html><head><title>Ergebnis</title></head>');
   Answer.HTMLDoc.Add('<body><h1>UpLoad-Ergebnis</h1><hr>');
   Answer.HTMLDoc.Add('<p><#msg></p><hr>');
   Answer.HTMLDoc.Add('<p align="Center"><small><#date>, <#time></small></p>');
   Answer.HTMLDoc.Add('</body></html>');
   Response.Content:=Answer.Content;                   // Notantwort generieren
  end;
end;

procedure TWebModul.AnswerHTMLTag(Sender: TObject; Tag: TTag;
  const TagString: String; TagParams: TStrings; var ReplaceText: String);
var sTag: String;
begin                   // Platzhaltertags <#msg>, <#date> und <#time> ersetzen
  sTag:=UpperCase(TagString);
  if sTag='TIME' then ReplaceText:=FormatDateTime('hh:nn:ss',Now())
  else if sTag='DATE' then ReplaceText:=FormatDateTime('dddd, yy-mm-dd',Now())
  else if sTag='MSG' then ReplaceText:=sMsg;
end;

end.
