unit PostContent; // Variante fr die Standard-Versionen von Delphi

{*****************************************************************************
 * enthlt die Funktion GetPostedContent(), die in direkt programmierten     *
 * Webapplikationen die mittels der Formularmethode POST geschickten Daten   *
 * entgegennimmt und verarbeitet.                                            *
 * Verarbeiten bedeutet:                                                     *
 * - alle Textfelder, Radiobuttons, Checkboxen, Buttons nach RFC 1867        *
 *   dekodieren und in Parameter "List" als name=wert eintragen              *
 * - alle <input type="file"> mit dem gesendeten Dateinamen unter Parameter  *
 *   "Path" auf dem Server abspeichern                                       *
 *                                                                           *
 * freigegeben:   Jrgen Hummel  *  www.hummel1.de  *  post@hummel1.de       *
 *                                                                           *
 * Stand: 01-07-20                                                           *
 *****************************************************************************}

{$IFNDEF CONSOLE} {$DEFINE ISAPI} {$ENDIF} {$H+}

interface

uses Windows, Classes, SysUtils {$IFDEF ISAPI}, Isapi {$ENDIF};

{$IFNDEF ISAPI}                                          // bei CGI-Anwendungen
function GetPostedContent(var List: TStringList; Path: TFileName): Boolean;
{$ELSE}                                                  // bei ISAPI-Projekten
function GetPostedContent(var List: TStringList; Path: TFileName; ECB: TExtension_Control_Block): Boolean;
{$ENDIF}

implementation

function HxToInt(c: Char): Byte; //Hilfsfunktion Dezimalwert eines Hex-Zeichens
begin if c<='9' then HxToInt:=Ord(c)-48 else HxToInt:=Ord(UpCase(c))-55 end;

function GetPostedContent(var List: TStringList; Path: TFileName {$IFDEF ISAPI}; ECB: TExtension_Control_Block {$ENDIF}): Boolean;
var sContent, sBuf, sField, sType, sBoundary, sBound2: String;
    nContLen, nP: Integer; nRead: {$IFNDEF VER100} Cardinal {$ELSE} Integer {$ENDIF};
    UpFile: TFileStream;
    {$IFNDEF ISAPI} pBuf: PChar; cBuf: array[0..255] of Char; {$ENDIF}
begin
 GetPostedContent:=true;
{$IFNDEF ISAPI} // nur bei CGI
 pBuf:=@cBuf; FillChar(cBuf,SizeOf(cBuf),#0);
 GetEnvironmentVariable('REQUEST_METHOD',pBuf,SizeOf(cBuf));
 if AnsiStrIComp(pBuf,'POST')=0 then begin                  // bei Methode POST
  FillChar(cBuf,SizeOf(cBuf),#0); GetEnvironmentVariable('CONTENT_LENGTH',pBuf,SizeOf(cBuf));
  if StrLen(pBuf)=0 then pBuf:=StrCopy(pBuf,'0');
  try nContLen:=StrToInt(cBuf) except nContLen:=0 end; nP:=1;
  FillChar(cBuf,SizeOf(cBuf),#0); GetEnvironmentVariable('CONTENT_TYPE',pBuf,SizeOf(cBuf)); sBuf:=cBuf;
  SetLength(sContent,nContLen);          // Pufferspeicher anlegen, dann fllen
  while nContLen>0 do begin
   ReadFile(GetStdHandle(STD_INPUT_HANDLE),sContent[nP],4096,nRead,nil); // keine greren Blcke mglich!!!
   Dec(nContLen,nRead); Inc(nP,nRead);
  end;
{$ELSE} // nur bei ISAPI
 if StrIComp(ECB.lpszMethod,'POST')=0 then begin            // bei Methode POST
  nContLen:=ECB.cbTotalBytes; nRead:=ECB.cbAvailable; sBuf:=ECB.lpszContentType;
  SetLength(sContent,nContLen);          // Pufferspeicher anlegen, dann fllen
  Move(ECB.lpbData^,sContent[1],nRead); nP:=nRead+1; Dec(nContLen,nRead);
  while nContLen>0 do begin
   nRead:=16384;                    // Blockgre evt. je nach Server austesten
   ECB.ReadClient(ECB.ConnID,Addr(sContent[nP]),nRead);
   Dec(nContLen,nRead); Inc(nP,nRead);
  end;
{$ENDIF} // ab hier gemeinsam fr ISAPI und CGI
  if Pos('multipart/form-data',LowerCase(sBuf))=1 then begin   // fr Form-Data
   if Path<>'' then if Path[Length(Path)]<>'\' then Path:=Path+'\'; // Speicher
   sBoundary:='--'+Copy(sBuf,Pos('boundary=',LowerCase(sBuf))+9,199);
   nP:=Pos(sBoundary,sContent); if nP=0 then GetPostedContent:=false;
   while nP>0 do begin                               // solange Feld-Abschnitte
    Delete(sContent,1,nP);                                  // bis dahin krzen
    sBuf:=Copy(sContent,1,255); nP:=Pos('name=',LowerCase(sBuf));
    if nP>0 then begin                            // Feldname "name= " gefunden
     sBuf:=Copy(sContent,nP,255);            // nchste 2..4 Zeilen ab "name= "
     sType:=Copy(sBuf,Pos(#10,sBuf)+1,255);             // 2. Zeile extrahieren
     Delete(sType,Pos(#13,sType),255);
     Delete(sBuf,Pos(#13,sBuf),255);        // Zeilenrest ab "name= " verbleibt
     sField:=Copy(sBuf,Pos('"',sBuf)+1,199);           // FeldNamen extrahieren
     Delete(sField,Pos('"',sField),199);
     if (Pos('/mixed',LowerCase(sType))>0) and (Pos('boundary=',LowerCase(sType))>0) then begin
      sBound2:='--'+Copy(sType,Pos('boundary=',LowerCase(sType))+9,199);
      sType:='';                                 // fr Dateinamen missbrauchen
      nP:=Pos(sBound2,sContent);             // erste eingebettete Datei suchen
      while nP>0 do begin
       Delete(sContent,1,nP);                               // bis dahin krzen
       sBuf:=Copy(sContent,1,255);                       // Dateinamen gewinnen
       Delete(sBuf,1,Pos('filename=',LowerCase(sBuf))); Delete(sBuf,1,Pos('=',sBuf));
       Delete(sBuf,1,Pos('"',sBuf)); if Pos('"',sBuf)>0 then Delete(sBuf,Pos('"',sBuf),199);
       if Length(sType)>0 then sType:=sType+'; '; sType:=sType+sBuf; sBuf:=ExtractFilename(sBuf);
       nP:=Pos(#13#10#13#10,sContent); Delete(sContent,1,nP+3);  // Dateianfang
       nP:=Pos(sBound2,sContent)-2;           // Ende Dateiabschnitt ohne CR/LF
       if sBuf>'' then try                        // unter Dateinamen speichern
        UpFile:=TFileStream.Create(Path+sBuf,fmOpenWrite or fmCreate);
        try UpFile.Write(sContent[1],nP-1) finally UpFile.Free end;
       except GetPostedContent:=false end;
       if Copy(sContent,nP+2+Length(sBound2),2)='--'then nP:=0; // Ende multipart/mixed
      end;
      sBuf:='';                        // Kennzeichen, dass multipart/mixed war
     end; // multipart/mixed
     nP:=Pos(#13#10#13#10,sContent); if nP>0 then Delete(sContent,1,nP+3); // Anfang Datenabschnitt
     nP:=Pos(sBoundary,sContent)-2;           // Ende Datenabschnitt ohne CR/LF
     if Pos('filename=',LowerCase(sBuf))=0 then begin         // kein Type File
      if sBuf<>'' then List.Add(sField+'='+Copy(sContent,1,nP-1)) //normalerweise
      else List.Add(sField+'='+sType);        // von /mixed kommende Dateinamen
     end else begin                                                // Type File
      Delete(sBuf,1,Pos('filename=',LowerCase(sBuf))+8); // Dateinamen gewinnen
      Delete(sBuf,1,Pos('"',sBuf)); if Pos('"',sBuf)>0 then Delete(sBuf,Pos('"',sBuf),199);
      List.Add(sField+'='+sBuf); sBuf:=ExtractFilename(sBuf);
      if sBuf>'' then try                         // unter Dateinamen speichern
       UpFile:=TFileStream.Create(Path+sBuf,fmOpenWrite or fmCreate);
       try if nP>1 then UpFile.Write(sContent[1],nP-1) finally UpFile.Free end;
      except GetPostedContent:=false end;
     end;
    end else begin Delete(sContent,1,1); nP:=Pos(sBoundary,sContent) end; // kein FeldName
   end; // while Pos(sBoundary)
  end else begin                                // bei www-form-urlencoded-Data
   nContLen:=Length(sContent); sBuf:=''; nP:=1;
   while nP<=nContLen do begin                           // Zeichen fr Zeichen
    case sContent[nP] of                             // Sonderzeichen behandeln
     '+': sBuf:=sBuf+' ';
     '%': begin sBuf:=sBuf+Chr((HxToInt(sContent[nP+1])shl 4)+HxToInt(sContent[nP+2])); Inc(nP,2) end;
     '&': begin List.Add(sBuf); sBuf:='' end;
     else sBuf:=sBuf+sContent[nP];                             // Normalzeichen
    end;
    Inc(nP);                                        // nchstes Eingabe-Zeichen
   end;
   List.Add(sBuf);
  end;                                                  // Ende Datenbehandlung
 end else GetPostedContent:=false;                   // Methode anders als POST
end;

end.
